/* gnubiff -- a mail notification program
 * Copyright (c) 2000-2004 Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include <sstream>
#include <cstdio>
#include <string>
#include <glib.h>
#include "Applet.h"
#include "Mailbox.h"
#include "Popup.h"



// ===================================================================
// = Static attributes ===============================================
// ===================================================================


// ===================================================================
// = Callbacks "C" binding ===========================================
// ===================================================================
extern "C" {
	//! C binding
	void APPLET_on_button_press (GtkWidget *widget, GdkEventButton *event, gpointer data)	{((Applet *) data)->on_button_press (event);}
	//! C binding
	gboolean APPLET_poll_timeout (gpointer data)	{return ((Applet *) data)->poll_timeout ();}
	gpointer APPLET_check (gpointer data)			{return ((Applet *) data)->check(FALSE);}
	gpointer APPLET_force_check (gpointer data)		{return ((Applet *) data)->check(TRUE);}
}


// ===================================================================
// = Applet =========================================================
// ===================================================================
Applet::Applet (Biff *owner, string xmlFilename) : GUI (xmlFilename) {
	_owner = owner;
	g_static_mutex_lock (&_timer_mutex);
	_polltag = 0;
	g_static_mutex_unlock (&_timer_mutex);
}


// ===================================================================
// = ~Applet ========================================================
// ===================================================================
Applet::~Applet (void) {
}

// ===================================================================
// = mutex ===========================================================
// ===================================================================
GStaticMutex Applet::_update_mutex = G_STATIC_MUTEX_INIT;
GStaticMutex Applet::_timer_mutex  = G_STATIC_MUTEX_INIT;


// ===================================================================
// = start ===========================================================
/**
   Start poll timer
*/
// ===================================================================
void Applet::start (void) {
	stop();
	g_static_mutex_lock (&_timer_mutex);
	_polltag = g_timeout_add (_owner->_polltime*1000, APPLET_poll_timeout, this);
	if (_owner->_debug)
		g_message ("POLL TAG : %d", _polltag);
	g_static_mutex_unlock (&_timer_mutex);
}


// ===================================================================
// = stop ============================================================
/**
   Stop poll timer
*/
// ===================================================================
void Applet::stop (void) {
	g_static_mutex_lock (&_timer_mutex);
	if (_polltag > 0)
		g_source_remove (_polltag);
	_polltag = 0;
	g_static_mutex_unlock (&_timer_mutex);
}


// ===================================================================
// = poll_timeout ====================================================
/**
   Callback on poll timeout
   @return false 
*/
// ===================================================================
gboolean Applet::poll_timeout (void) {
	if (_owner->_debug)
		g_message ("Thread created on APPLET_check (from Applet::poll_timeout)");
	g_thread_create (APPLET_check, this, FALSE, 0);
	return false;
}


// ===================================================================
// = check ===========================================================
/**
   Check for mail
   @param force indicates if this is a forced check (then no sound if new mail) or not
   @return 0
*/
// ===================================================================
gpointer Applet::check (gboolean force) {
	// If we can't get lock, it means that another poll is running
	// No need to make another one
	if (_owner->_debug)
		g_message ("LOCK update_mutex (Applet::check)");

	if (!g_static_mutex_trylock (&_update_mutex)) {
		if (_owner->_debug)
			g_message ("LOCK failed on update_mutex (Applet::check)\n");
		return 0;
	}
  
	// Stop timer
	stop();

	_owner->mailbox()->update();
	if (_owner->mailbox()->status() == MAILBOX_ERROR) {
		gdk_threads_enter();
		update();
		gdk_threads_leave();
		if (_owner->_state == STATE_RUNNING)
			start();
		g_static_mutex_unlock (&_update_mutex);
		if (_owner->_debug)
			g_message ("UNLOCK update_mutex (Applet::check)\n");
		return 0;
	}
	else if ((!force) && (_owner->mailbox()->status() == MAILBOX_OLD)) {
		if (_owner->_state == STATE_RUNNING)
			start();
		g_static_mutex_unlock (&_update_mutex);
		if (_owner->_debug)
			g_message ("UNLOCK update_mutex (Applet::check)\n");
		return 0;
	}

	_owner->mailbox()->fetch();

	if ((_owner->mailbox()->status() == MAILBOX_NEW) && (!force)) {
		// Play sound
		if (_owner->_sound_type == SOUND_BEEP) {
			gdk_threads_enter();
			gdk_beep ();    
			gdk_threads_leave();
		}
		else if (_owner->_sound_type == SOUND_FILE) {
			stringstream s;
			s << _owner->_sound_volume/100.0;
			string command = "play \"" + _owner->_sound_file + string ("\" -v ");
			command += s.str();
			command += " &";
			system (command.c_str());
		}
	}

	gdk_threads_enter();
	update();
	gdk_threads_leave();

	// Check if we display popup window depending on force value:
	//  force=false: display popup only if new mail and popup option is on
	//  force=true:  display popup if some mail (new or old)
	if (_owner->mailbox()->unreads() &&	((_owner->mailbox()->status() == MAILBOX_NEW) || (force && (_owner->mailbox()->status() == MAILBOX_OLD)))) {
		gdk_threads_enter();
		_owner->popup()->update();
		_owner->popup()->show();
		gdk_threads_leave();
		g_static_mutex_unlock (&_update_mutex);
		if (_owner->_debug)
			g_message ("UNLOCK update_mutex (Applet::check)\n");

		// He we do not start timer again since a popup will be raised.
		// (timer will be start again when popup pops down)
		return 0;
	}

	// Start timer again
	if (_owner->_state == STATE_RUNNING)
		start();

	g_static_mutex_unlock (&_update_mutex);
	if (_owner->_debug)
		g_message ("UNLOCK update_mutex (Applet::check)\n");
	return 0;
}


// ===================================================================
// = dock ============================================================
/**
   Dock function that will specialized later
*/
// ===================================================================
void Applet::dock (GtkWidget *widget) {}
